"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const history_1 = require("history");
const _1 = require(".");
// tslint:disable-next-line: no-commented-code
// tslint:disable: no-expression-statement
// tslint:disable: object-literal-sort-keys
// tslint:disable: no-duplicate-string
// tslint:disable: no-object-mutation
// HACK: wait for history wrapper to update DOM.
const waitForDomUpdate = () => new Promise(resolve => setTimeout(resolve));
beforeEach(() => {
    // Clear previous test's DOM.
    window.document.body.innerHTML = "";
    window.document.title = "";
});
describe("oaf-react-router", () => {
    test("doesn't throw when wrapping and unwrapping history", () => {
        const history = history_1.createBrowserHistory();
        const unwrap = _1.wrapHistory(history);
        unwrap();
    });
    test("sets the document title", () => __awaiter(void 0, void 0, void 0, function* () {
        const history = history_1.createBrowserHistory();
        _1.wrapHistory(history, {
            setPageTitle: true,
            documentTitle: () => "test title",
        });
        expect(document.title).toBe("");
        history.push("/");
        yield waitForDomUpdate();
        expect(document.title).toBe("test title");
    }));
    test("does not set the document title when setPageTitle is false", () => __awaiter(void 0, void 0, void 0, function* () {
        const history = history_1.createBrowserHistory();
        _1.wrapHistory(history, {
            setPageTitle: false,
            documentTitle: () => "test title",
        });
        expect(document.title).toBe("");
        history.push("/");
        yield waitForDomUpdate();
        expect(document.title).toBe("");
    }));
    test("leaves focus alone when repairFocus is false", () => __awaiter(void 0, void 0, void 0, function* () {
        const history = history_1.createBrowserHistory();
        _1.wrapHistory(history, { repairFocus: false });
        const main = document.createElement("main");
        const mainH1 = document.createElement("h1");
        main.append(mainH1);
        const randomButton = document.createElement("button");
        main.append(randomButton);
        document.body.append(main);
        randomButton.focus();
        expect(document.activeElement).toBe(randomButton);
        history.push("/");
        yield waitForDomUpdate();
        expect(document.activeElement).toBe(randomButton);
    }));
    test("moves focus to body when primary focus target cannot be focused", () => __awaiter(void 0, void 0, void 0, function* () {
        const history = history_1.createBrowserHistory();
        _1.wrapHistory(history);
        const main = document.createElement("main");
        const mainH1 = document.createElement("h1");
        // tslint:disable-next-line: no-empty
        mainH1.focus = () => { };
        main.append(mainH1);
        const randomButton = document.createElement("button");
        main.append(randomButton);
        document.body.append(main);
        randomButton.focus();
        expect(document.activeElement).toBe(randomButton);
        history.push("/");
        yield waitForDomUpdate();
        expect([document.body, document.documentElement]).toContain(document.activeElement);
    }));
    test("moves focus to the primary focus target", () => __awaiter(void 0, void 0, void 0, function* () {
        const history = history_1.createBrowserHistory();
        _1.wrapHistory(history);
        const main = document.createElement("main");
        const mainH1 = document.createElement("h1");
        main.append(mainH1);
        document.body.append(main);
        expect([document.body, document.documentElement]).toContain(document.activeElement);
        history.push("/");
        yield waitForDomUpdate();
        expect(document.activeElement).toBe(mainH1);
    }));
});
